using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Runtime.Remoting;
using System.Threading;

using Belikov.GenuineChannels;
using Belikov.GenuineChannels.DirectExchange;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineTcp;
using Belikov.GenuineChannels.TransportContext;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.Messaging;

namespace Client
{
	/// <summary>
	/// ChatClient demostrates simple client application.
	/// </summary>
	class ChatClient
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// wait for the server
			Console.WriteLine("Sleeping for 3 seconds.");
			Thread.Sleep(TimeSpan.FromSeconds(3));

			// setup .NET Remoting
			Console.WriteLine("Configuring network environment...");
			GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);

			Hashtable properties = new Hashtable();

			_transportContext = TransportContextServices.CreateDefaultTcpContext(properties, null);
//			GenuineTcpChannel genuineTcpChannel = new GenuineTcpChannel(properties, null, null);
//			_transportContext = genuineTcpChannel.ITransportContext;
			_transportContext.DirectExchangeManager.RegisterServerService("ReceiveMessage", new MessageReceiver());

			string objectURI;
			_url = GenuineUtility.Parse(ConfigurationSettings.AppSettings["RemoteHostUri"], out objectURI);

			Console.WriteLine("Please enter a nickname:");
			Nickname = Console.ReadLine();

			for ( ; ; )
			{
				try
				{
					Login();
					for ( ; ; )
					{
						Console.WriteLine("Enter an empty string to exit. Enter a message to send it to all clients:");
						string message = Console.ReadLine();
						if (message.Length <= 0)
							return ;

						SendMessage(message);
					}
				}
				catch(Exception ex)
				{
					Console.WriteLine("Exception: {0}.", ex.Message);
				}
			}
		}

		public static string Nickname;
		private static ITransportContext _transportContext;
		private static string _url;

		/// <summary>
		/// Handles network events.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">Event parameters.</param>
		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, e.HostInformation  == null ? "<unknown>" : e.HostInformation.ToString());

			if (e.EventType == GenuineEventType.GeneralServerRestartDetected)
			{
				Login();
			}
		}

		/// <summary>
		/// Sends a nickname to server and subscribes for messages.
		/// </summary>
		public static void Login()
		{
			GenuineChunkedStream chunkedStream = new GenuineChunkedStream(false);
			BinaryWriter binaryWriter = new BinaryWriter(chunkedStream);
			binaryWriter.Write(Nickname);

			HostInformation remoteHost = _transportContext.KnownHosts[_url];
			_transportContext.DirectExchangeManager.SendSync(remoteHost, "Login", chunkedStream);
		}

		/// <summary>
		/// Sends a message to the remote host.
		/// </summary>
		/// <param name="message">The message to be sent to the remote host.</param>
		public static void SendMessage(string message)
		{
			GenuineChunkedStream chunkedStream = new GenuineChunkedStream(false);
			BinaryWriter binaryWriter = new BinaryWriter(chunkedStream);
			binaryWriter.Write(message);

			HostInformation remoteHost = _transportContext.KnownHosts[_url];
			_transportContext.DirectExchangeManager.SendSync(remoteHost, "SendMessage", chunkedStream);
		}

	}
}
