using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Runtime.Remoting;
using System.Threading;

using Belikov.GenuineChannels;
using Belikov.GenuineChannels.DirectExchange;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineTcp;
using Belikov.GenuineChannels.TransportContext;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.Messaging;

namespace Client
{
	/// <summary>
	/// Receives messages and outputs them on the console.
	/// </summary>
	public class MessageReceiver : IServerServiceEntry
	{
		/// <summary>
		/// Process the message.
		/// </summary>
		/// <param name="stream">The content of the message.</param>
		/// <param name="sender">The remote host that sent this request.</param>
		/// <returns>Null stream.</returns>
		public Stream HandleMessage(Stream stream, HostInformation sender)
		{
			try
			{
				BinaryReader binaryReader = new BinaryReader(stream);
				string message = binaryReader.ReadString();
				string nickname = binaryReader.ReadString();

				Console.WriteLine("{0} sent {1}.", nickname, message);
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception during receiving a message: {0}.", ex.Message);
			}
			finally
			{
				stream.Close();
			}

			return Stream.Null;
		}

	}
}
