using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Runtime.Remoting;
using System.Threading;

using Belikov.GenuineChannels;
using Belikov.GenuineChannels.DirectExchange;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineTcp;
using Belikov.GenuineChannels.TransportContext;
using Belikov.GenuineChannels.Logbook;
using Belikov.GenuineChannels.Messaging;

namespace Server
{
	/// <summary>
	/// Performs login.
	/// </summary>
	public class Login : IServerServiceEntry
	{
		/// <summary>
		/// Constructs an instance of the Login class.
		/// </summary>
		/// <param name="chatRoom">The chat room.</param>
		public Login(ChatRoom chatRoom)
		{
			this.ChatRoom = chatRoom;
		}

		public ChatRoom ChatRoom;

		/// <summary>
		/// Process the message.
		/// </summary>
		/// <param name="stream">The content of the message.</param>
		/// <param name="sender">The remote host that sent this request.</param>
		/// <returns>Null stream.</returns>
		public Stream HandleMessage(Stream stream, HostInformation sender)
		{
			try
			{
				BinaryReader binaryReader = new BinaryReader(stream);
				string nickname = binaryReader.ReadString();
				this.ChatRoom.SubscribeClient(nickname);

				Console.WriteLine("Client with nickname {0} was subscribed to chat events.", nickname);
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception during receiving a message: {0}.", ex.Message);
			}
			finally
			{
				stream.Close();
			}

			return Stream.Null;
		}

	}
}
